package planet.behaviour;
import planet.commonapi.RouteMessage;
import planet.commonapi.Node;

/**
 * This class is used to invoke a behaviour. It acts 
 * like a facade calling a behaviour only when several preconditions are met. 
 * Pre-conditions on this default implementation are probability-based.  
 * @author Marc Sanchez
 */
public class BehaviourInvoker {
	/**
	 * Behaviour object: this properties is a placeholder for a behaviour.
	 */
	protected Behaviour beh;
	/**
	 * Behaviour property: probability to be checked.
	 */
	protected double prob;
	/*
	 * Randomizer.
	 */
	private java.util.Random random = new java.util.Random();

	// Default Constructor
	public BehaviourInvoker(Behaviour beh, double prob) {
		this.beh = beh;
		this.prob = prob;
	}
	/**
	 * This method invokes a behaviour. 
	 * @param msg The RouteMessage recently arrived.
	 * @param node The Node which holds the behaviour.
	 * @return Returns a BehaviourInvokerStatus with replies from the
	 * behaviour's invocation and a boolean signaling if the invocation
	 * was finally made.
	 */
	public BehaviourInvokerStatus invoke(RouteMessage msg, Node node) {
		if (random.nextDouble() <= prob) return new BehaviourInvokerStatus(beh.onMessage(msg, node), true);
		else return new BehaviourInvokerStatus(null, false);
	}
	/**
	 * @return Returns the name of a behaviour.
	 */
	public String getName() {
		return beh.getName();
	}
	/**
	 * @return Returns a string representation of the behaviour. In general, the toString
	 * method returns a string that "textually represents" this behaviour. The result should 
	 * be a concise but informative representation that is easy for a person to read. 
	 */ 
	public String toString() {
		return "Behaviour [" + this.beh + "] " +
			   "PreCondition On [" + this.prob + "]"; 
	}
}
