package planet.behaviour;

import planet.commonapi.exception.InitializationException;

/**
 * This interface provides a method to get a
 * an instance of a BehavioursPool, a singleton object which dispatches some
 * behaviours whenever a message pattern matches a pattern for a behaviour.
 * A pattern is an aggregation of the type and mode of a message. 
 * @author Marc Sanchez
 */
public interface BehavioursPoolFactory extends java.io.Serializable {
	
	/**
	 * Builds a BehaviourPool with the actual configuration on <b>Behaviour.properties</b> file.
	 * @return A reference to BehavioursPool.
	 * @throws InitializationException when an error occurs during the
	 * initialization of the Behaviours pool or when this factory method is
	 * nonapplicable.
	 */
	public BehavioursPool buildBehavioursPool() throws InitializationException;
}


