package planet.behaviour;

import planet.commonapi.exception.InitializationException;

/**
 * This class provides an implementation 
 * of BehavioursPoolFactory's interface.
 * @author Marc Sanchez
 */
public class BehavioursPoolFactoryImpl implements BehavioursPoolFactory {
	/*
	 * Behaviours Pool: Behaviours are stored on a pool addressed by patterns.
	 */
	protected BehavioursPool beh = null;
	
	public BehavioursPoolFactoryImpl(Class behavioursPoolClass, BehaviourProperties behProps, Class nodeClass) 
			throws InitializationException {
			Class[] interfaces = behavioursPoolClass.getInterfaces();
			boolean behPoolInterface = false;
			for(int it = 0; it < interfaces.length; it++) 
				if (interfaces[it].getName().equals("planet.behaviour.BehavioursPool")) behPoolInterface = true;
			if (behPoolInterface) {
				Class[] args =  { 
						java.lang.Class.class,		// Filter Class
						java.util.Vector.class, 	// Patterns
						java.lang.Class.class 		// Node Class
								};
				try {
					java.lang.reflect.Constructor c = behavioursPoolClass.getDeclaredConstructor(args);
					Object[] o = { behProps.getFilter(), behProps.getPatterns(), nodeClass };
					beh = (BehavioursPool) c.newInstance(o);
				} catch(NoSuchMethodException e) {
					throw new InitializationException("BehavioursPool Class provided does not implements (java.util.Vector, java.lang.Class) constructor", e);
				} catch(Exception generic) {
					throw new InitializationException("BehavioursPool cannot be built up", generic);
				}
			} else throw new InitializationException("BehavioursPool Class provided does not implements BehavioursPool interface");
	}
	/**
	 * Builds a BehaviourPool with the actual configuration on <b>Behaviour.properties</b> file.
	 * @return A reference to BehavioursPool.
	 * @throws InitializationException when an error occurs during the
	 * initialization of the Behaviours pool or when this factory method is
	 * nonapplicable.
	 */
	public BehavioursPool buildBehavioursPool() throws InitializationException {
		return beh;
	}
}
