package planet.commonapi.factory;

import planet.commonapi.Application;
import planet.commonapi.EndPoint;
import planet.commonapi.Node;
import planet.commonapi.exception.InitializationException;

/**
 * This interface attempts to abstract building EndPoints, using the
 * pattern design Factory Method. For this, requires an instance of
 * Application and other one of Node.
 * @author Jordi Pujol
 */
public interface EndPointFactory {
	/**
	 * Builds a new EndPoint that relates the Application <b>app</b>
	 * with the underlying Node <b>node</b>.
	 * @param app Application to install at this node.
	 * @param node Node where will be installed the Application.
	 * @return A new instance of EndPoint that relates the Application
	 * and the Node. 
	 */
	public EndPoint buildEndPoint(Application app, Node node) throws InitializationException;
}
