package planet.commonapi.factory;

import planet.commonapi.*;
import planet.commonapi.exception.*;

/**
 * It's a factory for Nodes abstraction. It pretends build Nodes of any
 * class. It has got two parameters:
 * 
 * <ul>
 * <li><b>IdFactory:</b> It is used to generate the Ids for new Nodes.</li>
 * <li><b>Node's Class:</b> It specifies the class for the new 
 * instances of Nodes.</li>
 * </ul>
 * 
 * Theese two parameters are fixed by properties file
 * or by use of the NodeFactory's methods. In the properties file exists the
 * following parameter's names:
 * <ul>
 * <li><b>DEFAULT_IDFACTORY</b>: It shows the class to use to generate new Ids.</li>
 * <li><b>DEFAULT_NODECLASS</b>: It shows the class to use to generate
 * new instances of Node.</li>
 * </ul>
 * IMPORTANT: The concrete implementation of Node must have one constructor:
 * <ol>
 * <li>The constructor with the Node's Id in parameter.</li>
 * <ol>  
 * @author Jordi Pujol
 * @see planet.commonapi.Node
 */
public interface NodeFactory extends java.io.Serializable {
	/**
	 * Builds a Node using the default IdFactory  and the default
	 * Class for him.
	 * @return A new instance of the default Node.
	 */
	public Node buildNode() throws InitializationException;
	
	/**
	 * Builds a Node using the <b>Id</b> that appears in parameter,
	 * using the default Class for him.
	 * @param id
	 * @return A new instance of the default Node with the specified Id.
	 */
	public Node buildNode(Id id) throws InitializationException;
	
	/**
	 * Returns the IdFactory that the concrete implementation uses to
	 * generate new Ids for new instances of Nodes.
	 * @return IdFactory which is used to generade new Ids.
	 */
	public IdFactory getIdFactory();
	
	/**
	 * Sets the IdFactory to use in the future, to generate new Ids.
	 * @param idFactory IdFactory to use to generate new Ids.
	 */
	public void setIdFactory(IdFactory idFactory);
	
	/**
	 * Gets Class for the generated Nodes.
	 * @return Class for the generated Nodes.
	 */
	public Class getNodeClass();
	
	/**
	 * Sets Class to generate new Nodes.
	 * @param nodeClass Class for the Nodes to build in the future.
	 */
	public void setNodeClass(Class nodeClass) throws InitializationException;
	
	/**
	 * Gets the actual topology for the network. It is necessary to known 
	 * how generate new Ids for the nodes.
	 * @return The topology of the network actually in use.
	 */
	public String getTopology();
	
	/**
	 * Sets the topology of the network to know how generates new Ids.
	 * @param topology Topology of the network.
	 */
	public void setTopology(String topology) throws InitializationException;
}
