package planet.generic.commonapi;

import planet.commonapi.NodeHandle;
import planet.commonapi.Id;

/**
 * NodeHandle for node. Being so simple his API, surely will be 
 * able to be reused for different nodes. It only mantain the Node Id
 * and a flag that informs if the related Node is alive.
 * 
 * @author <a href="mailto: jordi.pujol@estudiants.urv.es">Jordi Pujol</a>
 * Date: 13/05/2004
 */
public class NodeHandleImpl extends NodeHandle {
	/**
	 * Node Id.
	 */
	private Id nodeId = null;
	/**
	 * true if related Node is alive.
	 */
	private boolean alive = true;
	
	/**
	 * Builds a new NodeHandle for a Node,
	 * only identify its Id and sets its alive flag.
	 * @param nodeId Id of the related Node
	 * @param alive true if the related Node is alive. false
	 * in other case.
	 */
	public NodeHandleImpl (Id nodeId, boolean alive) {
		this.nodeId = nodeId;
		this.alive = alive;
	}

	/**
	 * Gets the Id of the related node
	 * @see planet.commonapi.NodeHandle#getId()
	 * @return Id of the node
	 */
	public Id getId() {
		return nodeId;
	}

	/**
	 * Inform if the related node is alive.
	 * @see planet.commonapi.NodeHandle#isAlive()
	 * @return true if the related node is alive. false
	 * in other case.
	 */
	public boolean isAlive() {
		return alive;
	}
	
	/**
	 * Updates the alive flag.
	 * @see planet.commonapi.NodeHandle#setAlive(boolean)
	 * @param alive New value for the alive flag.
	 */
	public void setAlive(boolean alive) {
		this.alive = alive;
	}
	
	/**
	 * Returns the String representation of this NodeHandle
	 * @see java.lang.Object#toString()
	 * @return String representation with the nodeId and alive flag.
	 */
	public String toString() {
		//return "{NodeHandleImpl, nodeId=["+nodeId+"], alive=["+alive+"]}";
		return ""+nodeId;
	}
	
	/**
	 * Always returns one as proximity between any two nodes.
	 * @see planet.commonapi.NodeHandle#getProximity()
	 * @return Always one
	 */
	public int getProximity() {
		return 1;
	}
	
	/**
	 * Overwrites this method to reflect the comparison between
	 * the related Id's. That is:
	 * <br>
	 * <center><b>return getId().equals(((NodeHandle)obj).getId());</b></center>
	 * @see java.lang.Object#equals(java.lang.Object)
	 * @param obj NodeHandle to be compared.
	 * @return true if and only if the related Id's are equals. false in
	 * other case.
	 */
	public boolean equals(Object obj) {
		return getId().equals(((NodeHandle)obj).getId());
	}
	
	/**
	 * Overwrites this method to reflect the hash code of the related Id.
	 * That is:
	 * <br>
	 * <center><b>return getId().hashCode();</b></center>
	 * @see java.lang.Object#hashCode()
	 * @return The hash code of the related Id.
	 */
	public int hashCode() {
		return getId().hashCode();
	}
}
