package planet.symphony.behaviours;

import planet.commonapi.Node;
import planet.commonapi.NodeHandle;
import planet.commonapi.RouteMessage;


/**
 * This behaviour routes RouteMessages towards RemoteNodes.
 * @author Marc Sanchez
 */
public class RoutingBehaviour implements planet.behaviour.Behaviour{
	/**
	 *  Given a RouteMessage and a Node as input, onMessage's method do execution 
	 * steps for the behaviour. Node should be casted to Node's class in order to
	 * manage data structures and protocol of designed overlay.
	 * @param msg RouteMessage taken as input.
	 * @param node Node taken as input.
	 * @return Returns either an array of RouteMessages or null when no messages
	 * need to transmit this node. 
	 */
	public RouteMessage[] onMessage(RouteMessage msg, Node node) {
		planet.symphony.SymphonyNode symphony = (planet.symphony.SymphonyNode) node;
		NodeHandle proposed = symphony.route(msg.getDestination());
		msg.setNextHopHandle(proposed);
		planet.results.LinkStateResults.updateOutcoming(symphony.getId());
		planet.results.LinkStateResults.updateHopsByTypeOf(msg);
		return new RouteMessage[] {msg};
	}
	/**
	 * @return Returns the name of behvaiour. 
	 */
	public String getName() {
		return "RoutingBehaviour";
	}
	/**
	 * @return Returns a string representation of the behaviour. In general, the toString
	 * method returns a string that "textually represents" this behaviour. The result should 
	 * be a concise but informative representation that is easy for a person to read. 
	 */ 
	public String toString() {
		return getName();
	}
}
