package planet.test.GML.GMLTopology;

import planet.commonapi.Network;
import planet.commonapi.exception.InitializationException;
import planet.generic.commonapi.GenericApp;
import planet.generic.commonapi.GenericAppWithGML;
import planet.generic.commonapi.GML.IdleGMLConstraint;
import planet.generic.commonapi.factory.GenericFactory;
import planet.util.Properties;



/**
 * <b> GMLTopologyTest </b> is an object class aimed at providing a 
 * description of a Chord Topology using The Graphic File Format GML
 * for its representations an according to the specific GMLConstraint 
 * used for edge selection.
 * @see planet.commonapi.GML.GMLConstraint for more details.  
 * @author  Marc Sanchez <marc.sanchez@estudiants.urv.es> 
 */
public class GMLTopologyTest extends GenericAppWithGML {
	  
	  // Constructor
	  public GMLTopologyTest () throws InitializationException {
	  	super("../conf/factories_gml.properties",
	  	        "../conf/sim.properties",
	  	        "../conf/chord.properties",
	  	        "../conf/GML.properties",
	  	        "../conf/GMLclasses.properties");
	  	try {
	  		System.out.println ("\n [Creating Network Of " + Properties.networkSize + " Nodes ]");
	  	    
	  	  	long start = System.currentTimeMillis();
	  	  	// Build network but not is stable
	  	  	Network network = GenericFactory.buildNetwork();
	  	  	// Wait until network is stable 
	  	  	int steps = network.stabilize();
	  	  	long stop = System.currentTimeMillis();
	  	    
	  	  	System.out.println (" [" + Properties.networkSize + " Nodes With [" + steps + "] Steps " +
	  	  						"	And [" + GenericApp.timeElapsedInSeconds(start,stop) + "] Seconds.\n");
	  	  	
	  	  	// Obtain Topology Information
	  	  	GenericFactory.generateGML(network, "network.gml", new IdleGMLConstraint(), true);
	        
	  	} catch (InitializationException e) {
	  		System.out.println("Occur an exception in initialization of " + this.getClass().getName()+ ": " + e.getMessage());
	  		System.exit(-1);
	  	}
	  }

	  /**
	   * Launches GMLTopologyTest app.
	   * @param args Ignore command line args.
	   */
	  public static void main (String args[]) {

	    try {
			GMLTopologyTest c = new GMLTopologyTest();
		} catch (InitializationException e) {
			System.out.println("Error during initialization of GMLTopologyTest");
			e.printStackTrace();
		}
	    System.exit(0);
	  }
}
